#embed-links .card-pf-body {
    opacity: 0.3;
}
#embed-here {
    display: block;
    width: 1024px;
    height: 768px;
}
#embed-address {
    display: block;
    margin-bottom: 20px;
}
a[target] {
    cursor: pointer;
}
iframe {
    display: block;
    width: 100%;
    height: 100%;
    border: 1px solid #ddd;
}
