from .testlib import Browser

# used in cockpit-machines, cockpit-podman, cockpit-certificates
Browser.get_checked

# kept as being potentially useful in the future
Browser.upload_file
Browser.wait_attr_not_contains
