body.login-pf {
    background-size: cover;
    background-color: #777777;
}

#badge {
    width: 225px;
    height: 80px;
    background-image: url("logo.png");
    background-size: contain;
    background-repeat: no-repeat;
}

#brand {
    font-size: 18pt;
    text-transform: uppercase;
}

#brand:before {
    content: "${NAME}";
}

#index-brand:before {
    content: "${NAME}";
}
