from cockpit.internal_endpoints import User
from cockpit.misc.print import Printer

# D-Bus properties
User.full
User.groups
User.home
User.id
User.shell

# getattr()
Printer.dbus_call
Printer.help
Printer.wait
